import json

from typer.testing import CliRunner

from mm_solana.account import get_public_key
from mm_solana.cli.cli import app

runner = CliRunner()


def test_new_cmd():
    res = runner.invoke(app, "wallet new -l 11")
    assert res.exit_code == 0

    accounts = json.loads(res.stdout)
    assert len(accounts) == 11
    for address, private in accounts.items():
        assert address == get_public_key(private)


def test_new_generates_different_keys():
    res1 = runner.invoke(app, "wallet new -l 2")
    assert res1.exit_code == 0

    res2 = runner.invoke(app, "wallet new -l 2")
    assert res2.exit_code == 0

    assert res1.stdout != res2.stdout



import sys

from mm_std import BaseConfig, print_json


def print_config_and_exit(exit_: bool, config: BaseConfig, exclude: set[str] | None = None) -> None:
    if exit_:
        print_json(config.model_dump(exclude=exclude))
        sys.exit(0)



from mm_solana import token
from mm_solana.account import generate_account


def test_get_balance(mainnet_node, usdt_token_address, usdt_owner_address, random_proxy):
    res = token.get_balance(mainnet_node, usdt_owner_address, usdt_token_address, proxy=random_proxy)
    assert res.unwrap() > 0


def test_get_balance_no_tokens_account_1(mainnet_node, usdt_token_address, random_proxy):
    res = token.get_balance(
        mainnet_node,
        generate_account().public_key,
        usdt_token_address,
        proxy=random_proxy,
        no_token_accounts_return_zero=False,
    )
    assert res.err == "no_token_accounts"


def test_get_balance_no_tokens_account_2(mainnet_node, usdt_token_address, random_proxy):
    res = token.get_balance(
        mainnet_node,
        generate_account().public_key,
        usdt_token_address,
        proxy=random_proxy,
    )
    assert res.ok == 0


def test_get_decimals(mainnet_node, usdt_token_address, random_proxy):
    res = token.get_decimals(mainnet_node, usdt_token_address, proxy=random_proxy)
    assert res.unwrap() == 6



import json

from typer.testing import CliRunner

from mm_solana.cli.cli import app

runner = CliRunner()


def test_new_cmd():
    public = "FkXw3cUycwsMYHAGacfhBJpa31WTvq5bUZNGhCW4GFVD"
    private_base58 = "3JTDggs2wpXWbQoL44XGHWzkAvC2LsfTVcx7gPX1dw7izxQ77oMvnMxCnCHQCGrM1RviAVpPhLexsjJKxqVq9Zqd"
    private_arr = "[115,21,127,168,159,11,95,157,134,159,18,133,4,114,119,129,96,140,193,211,35,139,32,177,172,204,5,2,193,94,194,122,219,42,145,4,32,80,123,141,81,235,64,60,55,47,81,102,41,253,60,85,116,238,173,120,98,38,211,48,253,89,18,196]"  # noqa: E501
    res = runner.invoke(app, f"wallet keypair {private_base58}")
    assert res.exit_code == 0

    keypair = json.loads(res.stdout)
    assert keypair["public"] == public
    assert keypair["private_base58"] == private_base58
    assert keypair["private_arr"] == private_arr



import random
from decimal import Decimal

from solana.rpc.api import Client
from solana.rpc.commitment import Commitment

from mm_solana.rpc import DEFAULT_MAINNET_RPC
from mm_solana.types import Nodes, Proxies


def lamports_to_sol(lamports: int, ndigits: int = 4) -> Decimal:
    return Decimal(str(round(lamports / 10**9, ndigits=ndigits)))


def get_node(nodes: Nodes | None = None) -> str:
    if nodes is None:
        return DEFAULT_MAINNET_RPC
    if isinstance(nodes, str):
        return nodes
    return random.choice(nodes)


def get_proxy(proxies: Proxies) -> str | None:
    if not proxies:
        return None
    if isinstance(proxies, str):
        return proxies
    return random.choice(proxies)


def get_client(
    endpoint: str,
    commitment: Commitment | None = None,
    extra_headers: dict[str, str] | None = None,
    proxy: str | None = None,
    timeout: float = 10,
) -> Client:
    return Client(endpoint, commitment=commitment, extra_headers=extra_headers, timeout=timeout, proxy=proxy)
